@@ -60,10 +60,13 @@ gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data
 
 static void reread_cd (gboolean ignore_no_media);
 static void update_ui_for_album (AlbumDetails *album);
+static void set_duplication (gboolean enable);
 
 /* Prototypes for the signal blocking/unblocking in update_ui_for_album */
 void on_title_edit_changed(GtkEditable *widget, gpointer user_data);
 void on_artist_edit_changed(GtkEditable *widget, gpointer user_data);
+void on_year_edit_changed(GtkEditable *widget, gpointer user_data);
+void on_disc_number_edit_changed(GtkEditable *widget, gpointer user_data);
 
 GladeXML *glade;
 
@@ -74,11 +77,12 @@ GConfClient *gconf_client;
 
 GtkWidget *main_window;
 static GtkWidget *message_area_vbox;
-static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry;
+static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_entry, *year_entry, *disc_number_entry;
 static GtkWidget *track_listview, *extract_button, *play_button;
 static GtkWidget *status_bar;
 static GtkWidget *extract_menuitem, *play_menuitem, *next_menuitem, *prev_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 static GtkWidget *submit_menuitem;
+static GtkWidget *duplicate, *eject;
 GtkListStore *track_store;
 static BaconMessageConnection *connection;
 GtkCellRenderer *toggle_renderer, *title_renderer, *artist_renderer;
@@ -92,6 +96,7 @@ gboolean strip_chars;
 gboolean eject_finished;
 gboolean open_finished;
 gboolean extracting = FALSE;
+static gboolean duplication_enabled;
 
 static gint total_no_of_tracks;
 static gint no_of_tracks_selected;
@@ -329,10 +334,10 @@ static void duration_cell_data_cb (GtkTreeViewColumn *tree_column,
 }
 
 static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
-				      GtkCellRenderer *cell,
-				      GtkTreeModel *tree_model,
-				      GtkTreeIter *iter,
-				      gpointer data)
+                                      GtkCellRenderer *cell,
+                                      GtkTreeModel *tree_model,
+                                      GtkTreeIter *iter,
+                                      gpointer data)
 {
   TrackState state;
   gtk_tree_model_get (tree_model, iter, COLUMN_STATE, &state, -1);
@@ -354,9 +359,10 @@ static void number_cell_icon_data_cb (GtkTreeViewColumn *tree_column,
 /* Taken from gedit */
 static void
 set_message_area_text_and_icon (GeditMessageArea *message_area,
-				const gchar   *icon_stock_id,
-				const gchar   *primary_text,
-				const gchar   *secondary_text)
+                                const gchar   *icon_stock_id,
+                                const gchar   *primary_text,
+                                const gchar   *secondary_text,
+                                GtkWidget *button)
 {
   GtkWidget *hbox_content;
   GtkWidget *image;
@@ -365,6 +371,9 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
   gchar *secondary_markup;
   GtkWidget *primary_label;
   GtkWidget *secondary_label;
+  AtkObject *ally_target;
+
+  ally_target = gtk_widget_get_accessible (button);
 
   hbox_content = gtk_hbox_new (FALSE, 8);
   gtk_widget_show (hbox_content);
@@ -386,10 +395,13 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
   gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
   gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
   gtk_misc_set_alignment (GTK_MISC (primary_label), 0, 0.5);
+  atk_object_add_relationship (ally_target,
+                               ATK_RELATION_LABELLED_BY,
+                               gtk_widget_get_accessible (primary_label));
 
   if (secondary_text != NULL) {
     secondary_markup = g_markup_printf_escaped ("<small>%s</small>",
-						secondary_text);
+                                                secondary_text);
     secondary_label = gtk_label_new (secondary_markup);
     g_free (secondary_markup);
     gtk_widget_show (secondary_label);
@@ -397,42 +409,43 @@ set_message_area_text_and_icon (GeditMessageArea *message_area,
     gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
     gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
     gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
+    atk_object_add_relationship (ally_target,
+                                 ATK_RELATION_LABELLED_BY,
+                                 gtk_widget_get_accessible (secondary_label));
   }
 
   gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (message_area),
-				   hbox_content);
+                                   hbox_content);
 }
 
 /* Taken from gedit */
 static void
 set_message_area (GtkWidget *container,
-		  GtkWidget *message_area)
+                  GtkWidget *message_area)
 {
   if (current_message_area == message_area)
-      return;
-
+    return;
+  
   if (current_message_area != NULL)
     gtk_widget_destroy (current_message_area);
-
+  
   current_message_area = message_area;
-
+  
   if (message_area == NULL)
     return;
 
   gtk_box_pack_start (GTK_BOX (container),
-		      current_message_area,
-		      FALSE,
-		      FALSE,
-		      0);
+                      current_message_area,
+                      FALSE, FALSE, 0);
 
   g_object_add_weak_pointer (G_OBJECT (current_message_area),
-			     (gpointer)&current_message_area);
+                             (gpointer)&current_message_area);
 }
 
-static GtkWidget* musicbrainz_submit_message_area_new (char *title,
-						       char *artist)
+static GtkWidget*
+musicbrainz_submit_message_area_new (char *title, char *artist)
 {
-  GtkWidget *message_area;
+  GtkWidget *message_area, *button;
   char *primary_text;
 
   g_return_val_if_fail (title != NULL, NULL);
@@ -440,34 +453,36 @@ static GtkWidget* musicbrainz_submit_message_area_new (char *title,
 
   message_area = gedit_message_area_new ();
 
+  button = gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+                                          _("_Submit Album"),
+                                          GTK_RESPONSE_OK);
   gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-			      _("_Submit Album"),
-			      GTK_RESPONSE_OK);
-  gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-			      GTK_STOCK_CANCEL,
-			      GTK_RESPONSE_CANCEL);
+                                 GTK_STOCK_CANCEL,
+                                 GTK_RESPONSE_CANCEL);
 
   /* Translators: title, artist */
   primary_text = g_strdup_printf (_("Could not find %s by %s on MusicBrainz."), title, artist);
-
+  
   set_message_area_text_and_icon (GEDIT_MESSAGE_AREA (message_area),
-				  "gtk-dialog-info",
-				  primary_text,
-				  _("You can improve the MusicBrainz database by adding this album."));
-
+                                  "gtk-dialog-info",
+                                  primary_text,
+                                  _("You can improve the MusicBrainz database by adding this album."),
+                                  button);
+  
   g_free (primary_text);
-
+  
   return message_area;
 }
 
-static void musicbrainz_submit_message_area_response (GeditMessageArea *message_area,
-						      gint           response_id,
-						      gpointer       user_data)
+static void
+musicbrainz_submit_message_area_response (GeditMessageArea *message_area,
+                                          int response_id,
+                                          gpointer user_data)
 {
   if (response_id == GTK_RESPONSE_OK) {
     on_submit_activate (NULL, NULL);
   }
-
+  
   set_message_area (message_area_vbox, NULL);
 }
 
@@ -489,10 +504,14 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_entry_set_text (GTK_ENTRY (title_entry), "");
     gtk_entry_set_text (GTK_ENTRY (artist_entry), "");
     gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
+    gtk_entry_set_text (GTK_ENTRY (year_entry), "");
+    gtk_entry_set_text (GTK_ENTRY (disc_number_entry), "");
     gtk_label_set_text (GTK_LABEL (duration_label), "");
     gtk_widget_set_sensitive (title_entry, FALSE);
     gtk_widget_set_sensitive (artist_entry, FALSE);
     gtk_widget_set_sensitive (genre_entry, FALSE);
+    gtk_widget_set_sensitive (year_entry, FALSE);
+    gtk_widget_set_sensitive (disc_number_entry, FALSE);
     gtk_widget_set_sensitive (play_button, FALSE);
     gtk_widget_set_sensitive (play_menuitem, FALSE);
     gtk_widget_set_sensitive (extract_button, FALSE);
@@ -501,6 +520,7 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
     gtk_widget_set_sensitive (prev_menuitem, FALSE);
     gtk_widget_set_sensitive (next_menuitem, FALSE);
+    set_duplication (FALSE);
 
     set_message_area (message_area_vbox, NULL);
   } else {
@@ -508,16 +528,28 @@ static void update_ui_for_album (AlbumDetails *album)
 
     g_signal_handlers_block_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_block_by_func (artist_entry, on_artist_edit_changed, NULL);
+    g_signal_handlers_block_by_func (year_entry, on_year_edit_changed, NULL);
+    g_signal_handlers_block_by_func (disc_number_entry, on_disc_number_edit_changed, NULL);
     gtk_entry_set_text (GTK_ENTRY (title_entry), album->title);
     gtk_entry_set_text (GTK_ENTRY (artist_entry), album->artist);
+    if (album->disc_number) {
+      gtk_entry_set_text (GTK_ENTRY (disc_number_entry), g_strdup_printf ("%d", album->disc_number));
+    }
+    if (album->release_date && g_date_valid (album->release_date)) {
+      gtk_entry_set_text (GTK_ENTRY (year_entry), g_strdup_printf ("%d", g_date_get_year (album->release_date)));
+    }
     g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
     g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
+    g_signal_handlers_unblock_by_func (year_entry, on_year_edit_changed, NULL);
+    g_signal_handlers_unblock_by_func (disc_number_entry, on_disc_number_edit_changed, NULL);
     /* Clear the genre field, it's from the user */
     gtk_entry_set_text (GTK_ENTRY (genre_entry), "");
 
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
     gtk_widget_set_sensitive (genre_entry, TRUE);
+    gtk_widget_set_sensitive (year_entry, TRUE);
+    gtk_widget_set_sensitive (disc_number_entry, TRUE);
     gtk_widget_set_sensitive (play_button, TRUE);
     gtk_widget_set_sensitive (play_menuitem, TRUE);
     gtk_widget_set_sensitive (extract_button, TRUE);
@@ -526,6 +558,7 @@ static void update_ui_for_album (AlbumDetails *album)
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
     gtk_widget_set_sensitive (prev_menuitem, FALSE);
     gtk_widget_set_sensitive (next_menuitem, FALSE);
+    set_duplication (TRUE);
     
     for (l = album->tracks; l; l=g_list_next (l)) {
       GtkTreeIter iter;
@@ -558,19 +591,18 @@ static void update_ui_for_album (AlbumDetails *album)
     if (album->metadata_source != SOURCE_MUSICBRAINZ) {
       GtkWidget *message_area;
 
-      message_area = musicbrainz_submit_message_area_new (album->title,
-							  album->artist);
+      message_area = musicbrainz_submit_message_area_new (album->title, album->artist);
 
       set_message_area (message_area_vbox, message_area);
 
       g_signal_connect (message_area,
-			"response",
-			G_CALLBACK (musicbrainz_submit_message_area_response),
-			NULL);
-
+                        "response",
+                        G_CALLBACK (musicbrainz_submit_message_area_response),
+                        NULL);
+      
       gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (message_area),
-					    GTK_RESPONSE_CANCEL);
-
+                                               GTK_RESPONSE_CANCEL);
+      
       gtk_widget_show (message_area);
     }
   }
@@ -789,11 +821,11 @@ static void audio_volume_changed_cb (GConfClient *client, guint cnxn_id, GConfEn
 {
   g_assert (strcmp (entry->key, GCONF_AUDIO_VOLUME) == 0);
   
- GtkWidget *volb = glade_xml_get_widget (glade, "volume_button");
- if (entry->value == NULL) {
+  GtkWidget *volb = glade_xml_get_widget (glade, "volume_button");
+  if (entry->value == NULL) {
     bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), 1.0);
   } else {
-  	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), gconf_value_get_float (entry->value));
+    bacon_volume_button_set_value (BACON_VOLUME_BUTTON (volb), gconf_value_get_float (entry->value));
   }
 }
 
@@ -887,8 +919,7 @@ static void reread_cd (gboolean ignore_no_media)
   
   /* Set watch cursor */
   if (realized) {
-    cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (main_window->window),
-					 GDK_WATCH);
+    cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (main_window->window), GDK_WATCH);
     gdk_window_set_cursor (main_window->window, cursor);
     gdk_cursor_unref (cursor);
     gdk_display_sync (gdk_drawable_get_display (main_window->window));
@@ -1053,6 +1084,10 @@ set_device (const char* device, gboolean ignore_no_media)
     if (tray_opened == FALSE) {
       reread_cd (ignore_no_media);
     }
+#if HAVE_NAUTILUS_BURN_DRIVE_CAN_EJECT
+    // Enable/disable the eject options based on wether the drive supports ejection
+    gtk_widget_set_sensitive (eject, nautilus_burn_drive_can_eject (drive));
+#endif
   }
 }
 
@@ -1106,8 +1141,7 @@ static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
   gboolean ignore_no_media = GPOINTER_TO_INT (user_data);
   g_assert (strcmp (entry->key, GCONF_DEVICE) == 0);
 
-  if (entry->value == NULL
-		  || !cd_drive_exists (gconf_value_get_string (entry->value))) {
+  if (entry->value == NULL || !cd_drive_exists (gconf_value_get_string (entry->value))) {
     device = prefs_get_default_device();
     if (device == NULL) {
 #ifndef IGNORE_MISSING_CD
@@ -1117,8 +1151,8 @@ static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_CLOSE,
                                        "<b>%s</b>\n\n%s",
-				       _("No CD-ROM drives found"),
-				       _("Sound Juicer could not find any CD-ROM drives to read."));
+                                       _("No CD-ROM drives found"),
+                                       _("Sound Juicer could not find any CD-ROM drives to read."));
       gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
@@ -1146,10 +1180,10 @@ static void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
     int response;
     
     dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-				     GTK_DIALOG_MODAL,
-				     GTK_MESSAGE_QUESTION,
-				     GTK_BUTTONS_NONE,
-				     _("The currently selected audio profile is not available on your installation."));
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_QUESTION,
+                                     GTK_BUTTONS_NONE,
+                                     _("The currently selected audio profile is not available on your installation."));
     gtk_dialog_add_button (GTK_DIALOG (dialog), "gtk-quit", GTK_RESPONSE_REJECT);
     gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Change Profile"), GTK_RESPONSE_ACCEPT);
     response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1237,10 +1271,10 @@ void on_submit_activate (GtkWidget *menuitem, gpointer user_data)
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_CLOSE,
                                        "<b>%s</b>\n\n%s\n%s: %s",
-				       _("Could not open URL"),
-				       _("Sound Juicer could not open the submission URL"),
-				       _("Reason"),
-				       error->message);
+                                       _("Could not open URL"),
+                                       _("Sound Juicer could not open the submission URL"),
+                                       _("Reason"),
+                                       error->message);
       gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_destroy (dialog);
@@ -1365,22 +1399,46 @@ void on_title_edit_changed(GtkEditable *widget, gpointer user_data) {
 void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
   GtkTreeIter iter;
   TrackDetails *track;
+  gchar *current_track_artist, *former_album_artist = NULL;
 
   g_return_if_fail (current_album != NULL);
+  
+  /* Unset the sortable artist field, as we can't change it automatically */
+  if (current_album->artist_sortname) {
+    g_free (current_album->artist_sortname);
+    current_album->artist_sortname = NULL;
+  }
+
   if (current_album->artist) {
-    g_free (current_album->artist);
+    former_album_artist = current_album->artist;
   }
   current_album->artist = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 
-  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &iter)) return;
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (track_store), &iter)) {
+    g_free (former_album_artist);
+    return;
+  }
   
   /* Set the artist field in each tree row */
   do {
-    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
-    g_free (track->artist);
-    track->artist = g_strdup (current_album->artist);
-    gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
+    gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_ARTIST, &current_track_artist, -1);
+    /* Change track artist if it matched album artist before the change */
+    if ((strcasecmp (current_track_artist, former_album_artist) == 0) || (strcasecmp (current_track_artist, current_album->artist) == 0)) {
+      gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
+      
+      g_free (track->artist);
+      track->artist = g_strdup (current_album->artist);
+      
+      if (track->artist_sortname) {
+        g_free (track->artist_sortname);
+        track->artist_sortname = NULL;
+      }
+      
+      gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
+    }
+  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (track_store), &iter));
+ 
+  g_free (former_album_artist);
 }
 
 void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
@@ -1391,6 +1449,34 @@ void on_genre_edit_changed(GtkEditable *widget, gpointer user_data) {
   current_album->genre = gtk_editable_get_chars (widget, 0, -1); /* get all the characters */
 }
 
+void on_year_edit_changed(GtkEditable *widget, gpointer user_data) {
+  const gchar* yearstr;
+  int year;
+  
+  g_return_if_fail (current_album != NULL);
+  
+  yearstr = gtk_entry_get_text (GTK_ENTRY (widget));
+  year = atoi (yearstr);
+  if (year > 0) {
+    if (current_album->release_date) {
+      g_date_set_dmy (current_album->release_date, 1, 1, year);
+    } else {
+      current_album->release_date = g_date_new_dmy (1, 1, year);
+    }
+  }
+}
+
+void on_disc_number_edit_changed(GtkEditable *widget, gpointer user_data) {
+    const gchar* discstr;  
+    int disc_number;
+
+    g_return_if_fail (current_album != NULL);
+    discstr = gtk_entry_get_text (GTK_ENTRY (widget));
+    disc_number = atoi(discstr);
+    if (disc_number > 0)
+        current_album->disc_number = disc_number;
+}
+
 void on_contents_activate(GtkWidget *button, gpointer user_data) {
   GError *error = NULL;
 
@@ -1418,7 +1504,7 @@ GtkWidget *
 sj_make_volume_button (void)
 {
   GtkWidget *w = bacon_volume_button_new (GTK_ICON_SIZE_SMALL_TOOLBAR,
-					  0.0, 1.0, 0.02);
+                                          0.0, 1.0, 0.02);
   bacon_volume_button_set_value (BACON_VOLUME_BUTTON (w), 1.0);
   return w;
 }
@@ -1452,6 +1538,81 @@ on_message_received (const char *message, gpointer user_data)
   }
 }
 
+/**
+ * Performs various checks to ensure CD duplication is available.
+ * If this is found TRUE is returned, otherwise FALSE is returned.
+ */
+static gboolean
+is_cd_duplication_available()
+{
+  // First check the nautilus-cd-burner tool is available in the path
+  gchar* nautilus_cd_burner = g_find_program_in_path ("nautilus-cd-burner");
+  if (nautilus_cd_burner == NULL) {
+    return FALSE;
+  } 
+  g_free(nautilus_cd_burner);
+
+  // Second check the cdrdao tool is available in the path
+  gchar* cdrdao = g_find_program_in_path ("cdrdao");
+  if (cdrdao == NULL) {
+    return FALSE;
+  } 
+  g_free(cdrdao);  
+
+  // Now check that there is at least one cd recorder available
+  GList                    *drives;
+  NautilusBurnDriveMonitor *monitor;
+
+  monitor = nautilus_burn_get_drive_monitor ();
+  drives = nautilus_burn_drive_monitor_get_recorder_drives (monitor);
+
+  if (drives == NULL) {
+     return FALSE;
+  }
+
+  g_list_free (drives);
+  return TRUE;
+}
+
+/**
+ * Clicked on duplicate in the UI (button/menu)
+ */
+void on_duplicate_activate (GtkWidget *button, gpointer user_data)
+{
+  GError *error = NULL;
+  const gchar* device;
+
+  device = nautilus_burn_drive_get_device (drive);
+  if (!g_spawn_command_line_sync (g_strconcat ("nautilus-cd-burner --source-device=", device, NULL), NULL, NULL, NULL, &error)) {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+                                       GTK_DIALOG_DESTROY_WITH_PARENT,
+                                       GTK_MESSAGE_ERROR,
+                                       GTK_BUTTONS_CLOSE,
+                                       "<b>%s</b>\n\n%s\n%s: %s",
+                                       _("Could not duplicate disc"),
+                                       _("Sound Juicer could not duplicate the disc"),
+                                       _("Reason"),
+                                       error->message);
+      gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      g_error_free (error);
+  }
+}
+
+/**
+ * Sets the duplication buttons sensitive property if duplication is enabled.
+ * This is setup in the main entry point.
+ */
+static void set_duplication(gboolean enabled)
+{
+  if (duplication_enabled) {
+    gtk_widget_set_sensitive (duplicate, enabled);
+  }
+}
+
 int main (int argc, char **argv)
 {
   GnomeProgram *program;
@@ -1478,9 +1639,9 @@ int main (int argc, char **argv)
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   g_option_context_set_ignore_unknown_options (ctx, TRUE);
 
-  program = gnome_program_init ("sound-juicer", 	 
-                      VERSION, LIBGNOMEUI_MODULE, 	 
-                      argc, argv, 	 
+  program = gnome_program_init ("sound-juicer",
+                      VERSION, LIBGNOMEUI_MODULE,
+                      argc, argv,
                       GNOME_PROGRAM_STANDARD_PROPERTIES,
                       GNOME_PARAM_GOPTION_CONTEXT, ctx,
                       NULL);
@@ -1499,9 +1660,9 @@ int main (int argc, char **argv)
   
   connection = bacon_message_connection_new ("sound-juicer");
   if (bacon_message_connection_get_is_server (connection) == FALSE) {
-	  bacon_message_connection_send (connection, RAISE_WINDOW);
-	  bacon_message_connection_free (connection);
-	  exit (0);
+    bacon_message_connection_send (connection, RAISE_WINDOW);
+    bacon_message_connection_free (connection);
+    exit (0);
   } else {
     bacon_message_connection_set_callback (connection, on_message_received, NULL);
   }
@@ -1559,6 +1720,8 @@ int main (int argc, char **argv)
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
   genre_entry = glade_xml_get_widget (glade, "genre_entry");
+  year_entry = glade_xml_get_widget (glade, "year_entry");
+  disc_number_entry = glade_xml_get_widget (glade, "disc_number_entry");
   track_listview = glade_xml_get_widget (glade, "track_listview");
   extract_button = glade_xml_get_widget (glade, "extract_button");
   extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
@@ -1567,6 +1730,8 @@ int main (int argc, char **argv)
   next_menuitem = glade_xml_get_widget (glade, "next_track_menuitem");
   prev_menuitem = glade_xml_get_widget (glade, "previous_track_menuitem");
   status_bar = glade_xml_get_widget (glade, "status_bar");
+  duplicate = glade_xml_get_widget (glade, "duplicate_menuitem");
+  eject = glade_xml_get_widget (glade, "eject");
 
   { /* ensure that the play/pause button's size is constant */
     GtkWidget *fake_button1, *fake_button2;
@@ -1577,14 +1742,14 @@ int main (int argc, char **argv)
     fake_button1 = gtk_button_new_from_stock (GTK_STOCK_MEDIA_PLAY);
     gtk_size_group_add_widget (size_group, fake_button1);
     g_signal_connect_swapped (play_button, "destroy",
-		    	      G_CALLBACK (gtk_widget_destroy),
-			      fake_button1);
-
+                              G_CALLBACK (gtk_widget_destroy),
+                              fake_button1);
+    
     fake_button2 = gtk_button_new_from_stock (GTK_STOCK_MEDIA_PAUSE);
     gtk_size_group_add_widget (size_group, fake_button2);
     g_signal_connect_swapped (play_button, "destroy",
-		    	      G_CALLBACK (gtk_widget_destroy),
-			      fake_button2);
+                              G_CALLBACK (gtk_widget_destroy),
+                              fake_button2);
 
     gtk_size_group_add_widget (size_group, play_button);
   }
@@ -1691,6 +1856,10 @@ int main (int argc, char **argv)
     return 0;
   }
 
+  // Set whether duplication of a cd is available using the nautilus-cd-burner tool
+  gtk_widget_set_sensitive (duplicate, FALSE);
+  duplication_enabled = is_cd_duplication_available();
+
   gconf_bridge_bind_window_size(gconf_bridge_get(), GCONF_WINDOW, GTK_WINDOW (main_window));
   gtk_widget_show (main_window);
   gtk_main ();
